package com.engine.attendance.attendanceanalysis.wrapper;

import com.engine.attendance.attendanceanalysis.service.ForgetClockInService;
import com.engine.attendance.attendanceanalysis.service.UtilService;
import com.engine.attendance.attendanceanalysis.service.impl.ForgetClockInServiceImpl;
import com.engine.attendance.attendanceanalysis.service.impl.UtilServiceImpl;
import com.engine.attendance.enums.CheckBoxEnum;
import com.engine.attendance.enums.ClassSegmentTypeEnum;
import com.engine.attendance.enums.ClockPointEnum;
import com.engine.common.util.DateUtil;
import com.engine.common.util.DbTools;
import com.engine.common.util.ServiceUtil;
import com.engine.common.util.Utils;
import com.engine.core.impl.Service;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import weaver.general.Util;

import java.time.ZoneOffset;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 考勤分析主题逻辑
 */
@Slf4j
public class AttendanceAnalysisWrapper extends Service {

    private UtilService utilService = ServiceUtil.getService(UtilServiceImpl.class);
    private ForgetClockInService forgetClockInService = ServiceUtil.getService(ForgetClockInServiceImpl.class);
    private UpdateAttendanceResultWrapper updateAttendanceResultWrapper = ServiceUtil.getService(UpdateAttendanceResultWrapper.class);

    /**
     * 考勤分析主逻辑入口
     * @param userId 人员id
     * @param dataList 打卡日期
     * @param attendanceItems 考勤项目
     * @param modeId 出勤结果formmodeid
     */
    public void attendanceAnalysis(String userId, List<Map<String,Object>> dataList,List<Map<String,Object>> attendanceItems,String modeId){
        log.info("***********analysis userId:{} start***********",userId);
        log.info("clockInTimeDate : [{}]",dataList);

        String beforeThreeDayDate = DateUtil.getCurrentDateMinusDay(3);
        String beforeTwoDayDate = DateUtil.getCurrentDateMinusDay(2);
        String beforeOneDayDate = DateUtil.getCurrentDateMinusDay(1);
        /** 打卡数据 */
        Map<String, List<Map<String,Object>>> collect = Maps.newHashMap();
        if (dataList != null && dataList.size() >0){
            collect = dataList.stream().collect(Collectors.groupingBy(e -> Util.null2String(e.get("signdate"))));
        }
        /** 获取人员班次*/
        Map<String,Object> classesParamMap = Maps.newHashMap();
        classesParamMap.put("tableName","uf_pbjg");
        classesParamMap.put("startDate",beforeThreeDayDate);
        classesParamMap.put("endDate",beforeOneDayDate);
        classesParamMap.put("pblx","0");
        classesParamMap.put("pbdx",userId);
        classesParamMap.put("current","1");
        classesParamMap.put("pageSize",10);
        classesParamMap.put("recurrence",1);
        Map<String,List<Map<String,Object>>> schedulingResultsMap = utilService.getSchedulingInFormation(classesParamMap);


        /**计算获取前一天和第二天打卡数据*/
        Map<String,Object> getClockTimeParam = Maps.newHashMap();
        getClockTimeParam.put("date",beforeOneDayDate);
        getClockTimeParam.put("clockInTimeCollect",collect);
        getClockTimeParam.put("schedulingResultCollect",schedulingResultsMap);
        //前一天打卡数据
        log.info("getClockTimeParam : [{}]",getClockTimeParam);
        List<Map<String, Object>> beforeOneDayClockInTimeList = utilService.getClockInTime(getClockTimeParam);
        log.info("beforeOneDayClockInTimeList : [{}]",beforeOneDayClockInTimeList);
        //分析前一天考勤
        analysis(userId,beforeOneDayDate,beforeOneDayClockInTimeList,schedulingResultsMap.get(beforeOneDayDate),attendanceItems,modeId);

        getClockTimeParam.put("date",beforeTwoDayDate);
        //前第二天打卡数据
        List<Map<String, Object>> beforeTwoDayClockInTimeList = utilService.getClockInTime(getClockTimeParam);
        log.info("beforeTwoDayClockInTimeList : [{}]",beforeTwoDayClockInTimeList);
        //分析前第二天考勤
        analysis(userId,beforeTwoDayDate,beforeTwoDayClockInTimeList,schedulingResultsMap.get(beforeTwoDayDate),attendanceItems,modeId);


    }

    /**
     * 考勤分析主逻辑入口
     * @param userId 人员id
     * @param analysisDate 分析日期
     * @param dataList 需要包含分析日期以及分析日期前后2天的3天打卡数据
     * @param attendanceItems 人员考勤项目
     * @param modeId 出勤结果formmodeid
     */
    public void attendanceAnalysisForApi(String userId,String analysisDate, List<Map<String,Object>> dataList,List<Map<String,Object>> attendanceItems,Map<String,List<Map<String,Object>>> schedulingResultsMap,String modeId){
        log.info("***********analysis userId:{} start***********",userId);
        log.info("clockInTimeDate : [{}]",dataList);
        Map<String, List<Map<String,Object>>> collect = Maps.newHashMap();
        if (dataList != null && dataList.size() >0){
            collect = dataList.stream().collect(Collectors.groupingBy(e -> Util.null2String(e.get("signdate"))));
        }
        /**计算获取前一天和第二天打卡数据*/
        Map<String,Object> getClockTimeParam = Maps.newHashMap();
        getClockTimeParam.put("date",analysisDate);
        getClockTimeParam.put("clockInTimeCollect",collect);
        getClockTimeParam.put("schedulingResultCollect",schedulingResultsMap);
        //前一天打卡数据
        log.info("getClockTimeParam : [{}]",getClockTimeParam);
        List<Map<String, Object>> clockInTimeList = utilService.getClockInTime(getClockTimeParam);

        analysis(userId,analysisDate,clockInTimeList,schedulingResultsMap.get(analysisDate),attendanceItems,modeId);

    }
    /**
     * 分析数据
     * @param userId 分析人员
     * @param analysisDate 分析日期
     * @param clockInTimeList 打卡数据
     * @param scheduleResult  班次
     * @param attendanceItems 考勤项目
     */
    public void analysis(String userId,String analysisDate,List<Map<String, Object>> clockInTimeList,List<Map<String, Object>> scheduleResult,List<Map<String,Object>> attendanceItems,String modeId){
        Map<String,Object> recordParam = Maps.newHashMap();
        recordParam.put("userId",userId);
        recordParam.put("analysisDate",analysisDate);
        recordParam.put("classInfo",scheduleResult);
        recordParam.put("modeId",modeId);

        if (scheduleResult == null || scheduleResult.size() ==0){

            //没有排班
            Map<String,Object> params = Maps.newHashMap();
            params.put("userId",userId);
            params.put("analysisDate",analysisDate);
            params.put("modeId",modeId);
            updateAttendanceResultWrapper.recordNoClass(params);
            return;
        }
        recordParam.put("rqlx",scheduleResult.get(0).get("rqlx"));
        List<Map<String, Object>> needClockInSchedule = scheduleResult.stream().filter(e -> "1".equals(e.get("ksdk")) || "1".equals(e.get("jsdk"))).collect(Collectors.toList());
        if(clockInTimeList == null || clockInTimeList.size() == 0){
            if (needClockInSchedule.size() > 0){
                //全天漏打卡
                recordParam.put("cqzt",0);
                recordParam.put("recordData",forgetClockInService.forgetClockInAllDays());
                utilService.recordItem(recordParam);
                return;
            }
        }
        //请假记录
        String sql = "select a.qjry,a.jqlx,a.cxjqj,b.ksrq,b.kssj,b.jsrq,b.jssj,b.qjsc,b.qtj,b.btj from uf_jcl_kq_qjjl a left join uf_jcl_kq_qjjl_dt1 b on a.id = b.mainid where a.qjry=? and b.ksrq=? and (b.cxqj=0 or b.cxqj is null)";
        List<Map<String,Object>> askForLeaveList = DbTools.getSqlToList(sql,userId,analysisDate);
        log.info("askForLeaveList :{}",askForLeaveList);
        //考勤项目
        Map<String,Map<String,Object>> askForLeaveItems = Maps.newHashMap();

        //出差记录
        sql = "select b.ccr,a.cclx,a.ccsd,b.ksrq,b.jsrq,b.kssj,b.jssj,b.ccsc,b.qtcc,b.btcc,b.cxcc from uf_jcl_kq_ccjl a left join uf_jcl_kq_ccjl_dt1 b on a.id = b.mainid where b.ccr=? and b.ksrq=? and (b.cxcc=0 or b.cxcc is null)";
        List<Map<String,Object>> evectionList = DbTools.getSqlToList(sql,userId,analysisDate);
        log.info("evectionList :{}",evectionList);
        //出差项目
        Map<String,Map<String,Object>> evectionItems = Maps.newHashMap();


        sql = "select hsl,hsdw,lgsbxydk,tqlghlfzs,fgsbxydk,thfghlfzs,tybcndbjlhbjs,zdycbcndfgzsd,zdycrqqjndxxb from uf_jcl_kq_kqxm where id=?";
        for (Map<String,Object> askForLeaveData : askForLeaveList){
            Map<String,Object> askForLeaveItem = DbTools.getSqlToMap(sql,askForLeaveData.get("jqlx"));
            //全天请假
            if (CheckBoxEnum.CHECKED.getKey().equals(askForLeaveData.get("qtj"))){
                List<Map<String,Object>> resultList = Lists.newArrayList();
                Map<String,Object> resultMap = Maps.newHashMap();
                resultMap.put("item",askForLeaveData.get("jqlx"));
                resultMap.put("itemduration",askForLeaveItem.get("hsl"));
                resultList.add(resultMap);
                recordParam.put("cqzt",CheckBoxEnum.CHECKED.getKey());
                recordParam.put("recordData",resultList);
                utilService.recordItem(recordParam);
                return;
            }

            askForLeaveItems.put(askForLeaveData.get("jqlx").toString(),askForLeaveItem);
            Map<String,Object> askForLeaveMap = new HashMap(){{
                put("bcxx",scheduleResult.get(0).get("bcxx"));
                put("bcsdxx",scheduleResult.get(0).get("bcsdxx"));
                put("edsc",scheduleResult.get(0).get("edsc"));
                put("bdlx", ClassSegmentTypeEnum.EVECTION.getKey());
                put("dtkssj",askForLeaveData.get("kssj"));
                put("dtjssj",askForLeaveData.get("jssj"));
                put("ksdk",askForLeaveItem.get("fgsbxydk"));
                put("jsdk",askForLeaveItem.get("lgsbxydk"));
                put("tqdkfzs","60");
                put("thdkfzs","60");
                put("sfdx","1");
                put("zddxfz",askForLeaveItem.get("tqlghlfzs"));
                put("dxhs",askForLeaveItem.get("thfghlfzs"));
            }};

            //scheduleResult.add(askForLeaveMap);
            if (!"".equals(Util.null2String(askForLeaveData.get("kssj"))) && !"".equals(Util.null2String(askForLeaveData.get("jssj"))) &&(CheckBoxEnum.CHECKED.getKey().equals(askForLeaveItem.get("lgsbxydk")) || CheckBoxEnum.CHECKED.getKey().equals(askForLeaveItem.get("fgsbxydk")))){
                needClockInSchedule.add(askForLeaveMap);
            }
        }
        for (Map<String,Object> evectionData :evectionList){
            //全天出差
            Map<String,Object> evectionItem = DbTools.getSqlToMap(sql,evectionData.get("cclx"));
            if (CheckBoxEnum.CHECKED.getKey().equals(evectionData.get("qtcc"))) {
                List<Map<String,Object>> resultList = Lists.newArrayList();
                Map<String,Object> resultMap = Maps.newHashMap();
                resultMap.put("item", evectionData.get("cclx"));
                resultMap.put("itemduration", evectionItem.get("hsl"));
                resultList.add(resultMap);
                recordParam.put("cqzt",CheckBoxEnum.CHECKED.getKey());
                recordParam.put("recordData",resultList);
                utilService.recordItem(recordParam);
                return;
            }
            evectionItems.put(evectionData.get("cclx").toString(),evectionItem);
            Map<String,Object> evectionMap = new HashMap(){{
                put("bcxx",scheduleResult.get(0).get("bcxx"));
                put("bcsdxx",scheduleResult.get(0).get("bcsdxx"));
                put("edsc",scheduleResult.get(0).get("edsc"));
                put("bdlx", ClassSegmentTypeEnum.ASK_FOR_LEAVE.getKey());
                put("dtkssj",evectionData.get("kssj"));
                put("dtjssj",evectionData.get("jssj"));
                put("ksdk",evectionItem.get("fgsbxydk"));
                put("jsdk",evectionItem.get("lgsbxydk"));
                put("tqdkfzs","60");
                put("thdkfzs","60");
                put("sfdx","1");
                put("zddxfz",evectionItem.get("tqlghlfzs"));
                put("dxhs",evectionItem.get("thfghlfzs"));
            }};
            //scheduleResult.add(evectionMap);
            if (!"".equals(Util.null2String(evectionData.get("kssj"))) && !"".equals(Util.null2String(evectionData.get("jssj"))) && (CheckBoxEnum.CHECKED.getKey().equals(evectionItem.get("lgsbxydk")) || CheckBoxEnum.CHECKED.getKey().equals(evectionItem.get("fgsbxydk")))){
                needClockInSchedule.add(evectionMap);
            }
        }

        needClockInSchedule = needClockInSchedule.stream().sorted(Comparator.comparing(e->DateUtil.getTime(analysisDate+" "+e.get("dtkssj")).toInstant(ZoneOffset.of("+8")).toEpochMilli())).collect(Collectors.toList());

        List<Map<String,Map<String,Object>>> clcokInTimeData = getClockInPoint(analysisDate,needClockInSchedule,clockInTimeList);

        Map<String,Object> recordAbnormalParam = Maps.newHashMap();
        recordAbnormalParam.put("clcokInTimeList",clcokInTimeData);
        recordAbnormalParam.put("attendanceItems",attendanceItems);
        recordAbnormalParam.put("scheduleResult",scheduleResult);
        recordAbnormalParam.put("analysisDate",analysisDate);
        recordAbnormalParam.put("userId",userId);
        recordAbnormalParam.put("rqlx",scheduleResult.get(0).get("rqlx"));
        recordAbnormalParam.put("modeId",modeId);

        log.info("recordAbnormalParam clcokInTimeList:{},attendanceItems size :{},scheduleResult :{},analysisDate:[{}],userId:[{}],rqlx:[{}],modeId:[{}]",clcokInTimeData,
                attendanceItems.size(),scheduleResult,analysisDate,userId,scheduleResult.get(0).get("rqlx"),modeId);

        /**
         * 正常上下班入结果表
         */
//
     //   List<Map<String,Object>> normalList= updateAttendanceResultWrapper.recordNormal(recordAbnormalParam);

        /**
         * 计算是否考勤异常,是否有早退、迟到、漏卡的情况
         * */
        List<Map<String,Object>> abnormalClockInList = updateAttendanceResultWrapper.recordAbnormalClockIn(recordAbnormalParam);


        /**
         * 计算加班
         */
        boolean recordWorkOverTime= updateAttendanceResultWrapper.recordWorkOverTime(recordAbnormalParam);

        /**
         * 请假
         */
        recordAbnormalParam.put("askForLeaveList",askForLeaveList);
        recordAbnormalParam.put("askForLeaveItems",askForLeaveItems);
        recordAbnormalParam.put("abnormalClockInList",abnormalClockInList);
        Map<String, Object> vactionMap = updateAttendanceResultWrapper.recordAskForLeave(recordAbnormalParam);
        abnormalClockInList = (List<Map<String,Object>>)vactionMap.get("abnormalClockInList");
        List<Map<String,Object>> vactionList = (List<Map<String,Object>>)vactionMap.get("resultList");
        /**
         * 出差、外出
         */
        recordAbnormalParam.put("evectionList",evectionList);
        recordAbnormalParam.put("evectionItems",evectionItems);
        recordAbnormalParam.put("abnormalClockInList",abnormalClockInList);

        Map<String, Object> recordEvection= updateAttendanceResultWrapper.recordEvection(recordAbnormalParam);
        abnormalClockInList = (List<Map<String,Object>>)recordEvection.get("abnormalClockInList");
        List<Map<String,Object>> evectionResultList = (List<Map<String,Object>>)recordEvection.get("resultList");

        /**
         * 入库
         */
        List<Map<String,Object>> recordData = Lists.newArrayList();
        recordData.addAll(vactionList);
        recordData.addAll(evectionResultList);
        recordData.addAll(abnormalClockInList);
        recordParam.put("recordData",recordData);
        recordParam.put("recordDataTime",utilService.getNeedRecordClockInTime(clcokInTimeData));
        log.info("recordParam : {}",recordParam);
        if (abnormalClockInList.size()>0){
            recordParam.put("cqzt",CheckBoxEnum.CHECKED.getKey());
        }else {
            recordParam.put("cqzt",CheckBoxEnum.UNCHECKED.getKey());
        }
        utilService.recordItem(recordParam);
    }


    /**
     * 获取打卡卡点
     * @param analysisDate
     * @param needClockInSchedule
     * @param clockInTimeList
     * @return
     */
    public List<Map<String,Map<String,Object>>> getClockInPoint(String analysisDate,List<Map<String, Object>> needClockInSchedule,List<Map<String, Object>> clockInTimeList){
        List<Map<String,Map<String,Object>>> clcokInTimeData = Lists.newArrayList();
        if (needClockInSchedule.size() > 0){
            String sfdx = Util.null2String(needClockInSchedule.get(0).get("sfdx"));
            String zddxfz = Util.null2String(needClockInSchedule.get(0).get("zddxfz"));
            String dxhs = Util.null2String(needClockInSchedule.get(0).get("dxhs"));
            if (CheckBoxEnum.CHECKED.getKey().equals(sfdx)){
                //上下班弹性
                List<Map<String, Object>> needClockInStart = needClockInSchedule.stream().filter(e -> ClassSegmentTypeEnum.WORK_TIME.getKey().equals(e.get("bdlx")) && CheckBoxEnum.CHECKED.getKey().equals(e.get("ksdk"))).collect(Collectors.toList());

                List<Map<String, Object>> needClockInEnd = needClockInSchedule.stream().filter(e -> ClassSegmentTypeEnum.WORK_TIME.getKey().equals(e.get("bdlx")) && CheckBoxEnum.CHECKED.getKey().equals(e.get("jsdk"))).collect(Collectors.toList());
            }
        }
        for (Map<String, Object> needClockIn :needClockInSchedule){
            if (CheckBoxEnum.CHECKED.getKey().equals(needClockIn.get("ksdk"))){
                String dtkssj = analysisDate+" "+needClockIn.get("dtkssj");
                String dtjssj = analysisDate+" "+needClockIn.get("dtjssj");
                if (DateUtil.getTime(dtkssj).compareTo(DateUtil.getTime(dtjssj)) > 0){
                    dtjssj = DateUtil.AfterDay(analysisDate,1) +" "+needClockIn.get("dtjssj");
                }
                int tqdkfzs = Integer.valueOf(Util.null2String(needClockIn.get("tqdkfzs")));
                Map<String,Map<String,Object>> ksdkNearestClcokInTime = Utils.getNearestClcokInTimeCmd(dtkssj,clockInTimeList);
                String timeType = ClockPointEnum.EMPTY.getKey();
                Map<String,Map<String,Object>> clcokInTimeMap = Maps.newHashMap();
                if (ksdkNearestClcokInTime.get(ClockPointEnum.EQUAL.getKey()) != null){
                    timeType=ClockPointEnum.EQUAL.getKey();
                }
                if (ClockPointEnum.EMPTY.getKey().equals(timeType) && ksdkNearestClcokInTime.get(ClockPointEnum.BEFORE.getKey()) != null){
                    String clockInTime = ksdkNearestClcokInTime.get(ClockPointEnum.BEFORE.getKey()).get("signdate") +" "+ksdkNearestClcokInTime.get(ClockPointEnum.BEFORE.getKey()).get("signtime");
                    if (DateUtil.getTime(clockInTime).compareTo(DateUtil.getTime(DateUtil.beforeMinutes(dtkssj,tqdkfzs))) >=0){
                        //打卡时间大于等于最早打卡时间
                        timeType=ClockPointEnum.BEFORE.getKey();
                    }
                }
                if (ClockPointEnum.EMPTY.getKey().equals(timeType) && ksdkNearestClcokInTime.get(ClockPointEnum.AFTER.getKey()) != null){
                    String clockInTime = ksdkNearestClcokInTime.get(ClockPointEnum.AFTER.getKey()).get("signdate") +" "+ksdkNearestClcokInTime.get(ClockPointEnum.AFTER.getKey()).get("signtime");
                    if (DateUtil.getTime(clockInTime).compareTo(DateUtil.getTime(dtjssj)) < 0){
                        //打卡时间小于结束时间
                        timeType=ClockPointEnum.AFTER.getKey();
                    }
                }
                clcokInTimeMap.put(dtkssj+"|"+ClockPointEnum.START.getKey()+"|"+timeType,ksdkNearestClcokInTime.get(timeType));
                clcokInTimeData.add(clcokInTimeMap);
            }
            if (CheckBoxEnum.CHECKED.getKey().equals(needClockIn.get("jsdk"))){
                String dtkssj = analysisDate+" "+needClockIn.get("dtkssj");
                String dtjssj = analysisDate+" "+needClockIn.get("dtjssj");
                int thdkfzs = Integer.valueOf(Util.null2String(needClockIn.get("thdkfzs")));
                if (DateUtil.getTime(dtkssj).compareTo(DateUtil.getTime(dtjssj)) > 0){
                    dtjssj = DateUtil.AfterDay(analysisDate,1) +" "+needClockIn.get("dtjssj");
                }
                Map<String,Map<String,Object>> jsdkNearestClcokInTime = Utils.getNearestClcokInTimeCmd(dtjssj,clockInTimeList);

                String timeType = ClockPointEnum.EMPTY.getKey();
                Map<String,Map<String,Object>> clcokInTimeMap = Maps.newHashMap();
                if (jsdkNearestClcokInTime.get(ClockPointEnum.EQUAL.getKey()) != null){
                    timeType=ClockPointEnum.EQUAL.getKey();
                }
                if (ClockPointEnum.EMPTY.getKey().equals(timeType) && jsdkNearestClcokInTime.get(ClockPointEnum.AFTER.getKey()) != null){
                    String clockInTime = jsdkNearestClcokInTime.get(ClockPointEnum.AFTER.getKey()).get("signdate") +" "+jsdkNearestClcokInTime.get(ClockPointEnum.AFTER.getKey()).get("signtime");
                    if (DateUtil.getTime(clockInTime).compareTo(DateUtil.getTime(DateUtil.AfterMinutes(dtjssj,thdkfzs))) <=0){
                        timeType=ClockPointEnum.AFTER.getKey();
                    }
                }
                if (ClockPointEnum.EMPTY.getKey().equals(timeType) && jsdkNearestClcokInTime.get(ClockPointEnum.BEFORE.getKey()) != null){
                    String clockInTime = jsdkNearestClcokInTime.get(ClockPointEnum.BEFORE.getKey()).get("signdate") +" "+jsdkNearestClcokInTime.get(ClockPointEnum.BEFORE.getKey()).get("signtime");
                    if (DateUtil.getTime(clockInTime).compareTo(DateUtil.getTime(dtkssj)) >0){
                        timeType=ClockPointEnum.BEFORE.getKey();
                    }
                }
                clcokInTimeMap.put(dtjssj+"|"+ClockPointEnum.END.getKey()+"|"+timeType,jsdkNearestClcokInTime.get(timeType));
                clcokInTimeData.add(clcokInTimeMap);
            }
        }
        //当有2笔需要打卡时，可能会有打卡歧义的情况，歧义情况取2个时间点的中间值，当打卡时间小于中间值归属前一个打卡，大于则相反
        if (clcokInTimeData.size() >1){
            for (int i=0;i<clcokInTimeData.size()-1;i++){
                Map<String,Map<String,Object>> beforeClcokInTimeData = clcokInTimeData.get(i);
                Map<String,Map<String,Object>> afterClcokInTimeData = clcokInTimeData.get(i+1);
                Map<String,Object> beforeClcokInTimeMap = null;
                String beforeClcokInTime = "";
                String afterClcokInTime = "";
                Map<String,Object> afterClcokInTimeMap = null;
                for (Map.Entry<String,Map<String,Object>> beforeEntry :beforeClcokInTimeData.entrySet()){
                    beforeClcokInTimeMap = beforeEntry.getValue();
                    beforeClcokInTime = beforeEntry.getKey();
                }
                for (Map.Entry<String,Map<String,Object>> afterEntry :afterClcokInTimeData.entrySet()){
                    afterClcokInTimeMap = afterEntry.getValue();
                    afterClcokInTime = afterEntry.getKey();
                }
                //重复
                if (beforeClcokInTimeMap != null && beforeClcokInTimeMap == afterClcokInTimeMap ){
                    String beforeTime = beforeClcokInTime.split("\\|")[0];
                    String afterTime = afterClcokInTime.split("\\|")[0];
                    long betWeenMinutes = DateUtil.getBetWeenMinutes(beforeTime,afterTime);

                    String middileTime = DateUtil.AfterMinutes(beforeTime,betWeenMinutes/2);

                    String signdateTime = beforeClcokInTimeMap.get("signdate") +" "+beforeClcokInTimeMap.get("signtime");

                    if (DateUtil.getTime(signdateTime).compareTo(DateUtil.getTime(middileTime)) <=0){
                        //该打卡归属前一个打卡点
                        afterClcokInTimeData.put(afterClcokInTime,null);
                    }else if (DateUtil.getTime(signdateTime).compareTo(DateUtil.getTime(middileTime)) >0){
                        //该打卡归属后一个打卡点
                        beforeClcokInTimeData.put(beforeClcokInTime,null);
                    }
                }
            }
        }

        return clcokInTimeData;
    }
}
